package abstractions;

import interfaces.LogicUnitForElevatorSpecification;
import interfaces.ElevatorForLogicUnitSpecification;

/**
 * A component of an {@link models.elevators.Elevator} system allowing dynamic binding of a controller to an elevator.<br>
 * Binding a controller to an elevator is code that is common to all concrete subclasses of the
 * {@link interfaces.LogicUnitForElevatorSpecification} interface, 
 * and so we implement it in an abstract superclass as concrete
 * method {@link  #bindToElevator(ElevatorForLogicUnitSpecification elevator)} . <br>
 * The concrete subclasses still have to implement the abstract methods:
 * <ul>
 * <li> {@link interfaces.LogicUnitForElevatorSpecification#calculateDirection()} </li>
 * <li> {@link interfaces.LogicUnitForElevatorSpecification#stopAtNextFloor()}</li>
 * </ul>
 * @author jpaulgibson
 */
public abstract class LogcUnitForElevatorBinding implements LogicUnitForElevatorSpecification{
	
	protected ElevatorForLogicUnitSpecification elevator;
	
	public LogcUnitForElevatorBinding (ElevatorForLogicUnitSpecification elevator){
		this.elevator = elevator;
	}
	
	public void bindToElevator(ElevatorForLogicUnitSpecification elevator) throws IllegalArgumentException{

		if (elevator == null) throw new IllegalArgumentException("Cannot bind controller to a null elevator");
		
		if (this.elevator != elevator){
		  this.elevator = elevator;
		  elevator.installLogicUnit(this);
		}
	}
	

}
