package enumerations;


/**
 * This logic type must be kept coherent with all the logic unit options
 * specified as subclasses of {@link abstractions.LogcUnitForElevatorBinding}:
 * <ul>
 * <li> {@link #RANDOM} - {@link models.elevatorLogicUnit.RandomMovementLogicUnit} </li>
 * <li> {@link #STUPID} - {@link models.elevatorLogicUnit.StupidLogicUnit}  </li>
 * <li> {@link #IMPROVED} - {@link models.elevatorLogicUnit.ImprovedLogicUnit}  </li>
 * <li> {@link #EMERGENCY} - {@link models.elevatorLogicUnit.EmergencyLogicUnit}  </li>
 * <li> {@link #SMART} - {@link models.elevatorLogicUnit.SmartLogicUnit}  </li> 
 * </ul>
 * It is also recommended that the {@link factoriesSPL.LogicUnitFactory} is kept up to date so that every
 * time a new logic unit type is added then the factory knows how to build a logic unit of that type.
 * @author jpaulgibson
 */
public enum LogicType {
	
	/**
	 * Controller type for {@link models.elevatorLogicUnit.RandomMovementLogicUnit}
	 */
	RANDOM, 
	
	/**
	 * Controller type for {@link models.elevatorLogicUnit.StupidLogicUnit}
	 */
	STUPID, 
	
	/**
	 * Controller type for {@link models.elevatorLogicUnit.ImprovedLogicUnit}
	 */
	IMPROVED, 
	
	/**
	 * Controller type for {@link models.elevatorLogicUnit.EmergencyLogicUnit}
	 */
	EMERGENCY, 
	
	/**
	 * Controller type for {@link models.elevatorLogicUnit.SmartLogicUnit}
	 */
	SMART

}