package enumerations;

/**
 * This security type must be kept coherent with all the security options
 * specified as subclasses of {@link abstractions.LockableFloorButtonsInElevator}:
 * <ul>
 * <li> {@link #NONE} - revert to superclass behaviour with no security - {@link models.elevatorButtons.FloorButtonsInElevator} </li>
 * <li> {@link #OPTION1} - {@link models.elevatorButtons.LockableFloorButtonsInElevator_Option1} </li>
 * <li> {@link #OPTION2} - {@link models.elevatorButtons.LockableFloorButtonsInElevator_Option2} </li>
 * </ul>
 * @author jpaulgibson
 */
public enum SecurityType {

	/**
	 * Security type for {@link models.elevatorButtons.FloorButtonsInElevator}, where there is no security offered
	 */
	NONE, 
	
	/**
	 * Security type for {@link models.elevatorButtons.LockableFloorButtonsInElevator_Option1}.<br>
	 * If there is  an existing floor request inside the elevator for the floor that has just been locked then
     * do not delete/remove this request. The request will be deleted automatically when the elevator arrives
     * at the floor.
	 */
	OPTION1, 
	
	/**
	 * Security type for {@link models.elevatorButtons.LockableFloorButtonsInElevator_Option2}.<br>
	 * If there is  an existing floor request inside the elevator for the floor that has just been locked then
     * delete/remove this request
	 */
	OPTION2
}
