package enumerations;



/**
 * This user type must be kept coherent with all the user simulation options
 * specified as subclasses of {@link abstractions.ElevatorUser}:
 * <ul>
 * <li> {@link #INTELLIGENT} - {@link models.elevatorUsers.IntelligentElevatorUser} </li>
 * <li> {@link #PRESS_ALL_BUTTONS} - {@link models.elevatorUsers.PressAllButtonsElevatorUser} </li>
 * <li> {@link #INTELLIGENT_WITH_KEY} - {@link models.elevatorUsers.IntelligentElevatorUserWithKey} </li>
 * </ul>
 * It is also recommended that the {@link factoriesSPL.ElevatorUserFactory} is kept up to date so that every
 * time a new user type is added then the factory knows how to build a user of that type.
 * @author jpaulgibson
 */
public enum UserType {
	
	/**
	 * Press only the buttons required to request the elevator at the floor (either up or down, but not both)
	 * and the floor destination button inside the elevator
	 */
	INTELLIGENT,
	
	/**
	 * Press all buttons at the floor (up and down) and all floor destination buttons inside the elevator
	 */
	PRESS_ALL_BUTTONS,
	
	/**
	 * The user has a key to unlock the destination floor to which they wish to travel (if it is needed because
	 * one of the {@link enumerations.SecurityType} options has been installed in the elevator).
	 */
	INTELLIGENT_WITH_KEY
	
}
