package models.elevatorLogicUnit;

import enumerations.LogicType;
import enumerations.DirectionState;
import interfaces.ElevatorForLogicUnitSpecification;

/**
 * The improved stupid controller ensures that the elevator moves from bottom to top to bottom to top to ...;
 * but its behaviour improves that of the {@link StupidLogicUnit} because it
 * stops only at floors where requests are made.
 * This corresponds to IMPROVED in the {@link enumerations.LogicType}
 * @author J Paul Gibson
 */
public class ImprovedLogicUnit extends StupidLogicUnit{

	
	public ImprovedLogicUnit(ElevatorForLogicUnitSpecification elevator) {
		super(elevator);
	}
	
	/**
	 * Only stop at floors where requests to stop have been made
	 */
	public boolean stopAtNextFloor(){
		int floor = elevator.currentFloor();
		DirectionState dir = elevator.currentDirection();
		
		if (elevator.floorPressedInElevator(floor)) return true;
		if (elevator.downPressedAtFloor(floor) && dir == DirectionState.DOWN) return true;
		if (elevator.upPressedAtFloor(floor) && dir == DirectionState.UP) return true;
		
		return false;
	}
	
	public String toString (){
		
		return LogicType.IMPROVED.toString();
	}
}
