package models.elevatorUsers;

import interfaces.ElevatorForUsersSpecification;
import interfaces.ElevatorWithLockableFloorsSpecification;

import java.util.Random;

import enumerations.DirectionState;
import enumerations.UserState;


public class IntelligentElevatorUserWithKey extends IntelligentElevatorUser{
	
public IntelligentElevatorUserWithKey (int startFloor, int endFloor, ElevatorForUsersSpecification elevator) throws IllegalArgumentException, IllegalStateException{
		
		super(startFloor, endFloor, elevator);	
	}
	
	public IntelligentElevatorUserWithKey (Random RNG, ElevatorForUsersSpecification elevator) throws IllegalArgumentException, IllegalStateException{
		
		super(RNG, elevator);		
	}

	public String toString (){
		String str = "IntelligentElevatorUserWithKey "+ unique_ID+ " - ";
		str = str + "from: " + startFloor;
		str = str + " to: "+  endFloor;
		str = str + " currently "+ state;
		return str;
	}
	
	@Override
	public void makeDecision() {
		
	        	// What direction does the user wish to travel?
				DirectionState user_direction;
				 if (startFloor < endFloor) user_direction = DirectionState.UP;
				 else user_direction = DirectionState.DOWN;
				 
				
				// Which floor button(s) does the user press when they arrive at the floor?
				if (state == UserState.ARRIVED_AT_FLOOR){
				if (user_direction == DirectionState.UP)
				     elevator.pressUp(startFloor);
				 else  
					 elevator.pressDown(startFloor);
				 state = UserState.WAITING_AT_FLOOR;
				}
				
				// Does the user get into the elevator and what buttons do they press?
				else if (startFloor == elevator.currentFloor() && 
		    		 state == UserState.WAITING_AT_FLOOR &&
		    		 user_direction == elevator.currentDirection() 
		    		 ){
		    			 pickedUpAtFloor();
		    			 
		    			 if ( elevator instanceof ElevatorWithLockableFloorsSpecification &&
		    				  ((ElevatorWithLockableFloorsSpecification) elevator).is_locked(endFloor))
		    			 ((ElevatorWithLockableFloorsSpecification) elevator).unlock(endFloor);
		    					 
		    			 elevator.pressFloorInElevator(endFloor); 
				}
	
		 // Does the user get out of the elevator?
		 else if (endFloor== elevator.currentFloor() &&
    		 state  == UserState.IN_ELEVATOR){
        	 
        	 deliveredToFloor();
         
        	 if ( elevator instanceof ElevatorWithLockableFloorsSpecification &&
        		  !((ElevatorWithLockableFloorsSpecification) elevator).is_locked(endFloor))
				  ((ElevatorWithLockableFloorsSpecification) elevator).lock(endFloor);
    	 }
	

	}
}
