package models.elevatorUsers;

import interfaces.ElevatorForUsersSpecification;
import enumerations.DirectionState;
import enumerations.UserState;

import java.util.Random;

import abstractions.ElevatorUser;


/**
 * Tested by:
 * <ul> 
 * <li> unit tests -  {@link unit_tests.elevatorUsers.JUnit_PressAllButtonsElevatorUser} </li>
 * <li> validation tests -  {@link validation_tests.Validation_PressAllButtonsElevatorUser} </li>
 * </ul>
 * This elevator user behaves intelligently as defined in {@link #makeDecision()}<br>
 * This specialisation corresponds to PRESS_ALL_BUTTONS in the {@link enumerations.UserType}
 * 
 * @author jpaulgibson
 */
public class PressAllButtonsElevatorUser extends ElevatorUser {
	

	public PressAllButtonsElevatorUser (int startFloor, int endFloor, ElevatorForUsersSpecification elevator) throws IllegalArgumentException, IllegalStateException{
		
		super(startFloor, endFloor, elevator);	
	}
	
	public PressAllButtonsElevatorUser (Random RNG, ElevatorForUsersSpecification elevator) throws IllegalArgumentException, IllegalStateException{
		
		super(RNG, elevator);		
	}

	
	public boolean equals (Object obj){
		
		if (obj == null) return false;
		if (this == obj) return true;
		if (! (obj instanceof PressAllButtonsElevatorUser)) return false;
		PressAllButtonsElevatorUser u = (PressAllButtonsElevatorUser) obj;
		return this.unique_ID == u.unique_ID;
		
	}
	
	public String toString (){
		String str = "PressAllButtonsElevatorUser "+ unique_ID+ " - ";
		str = str + "from: " + startFloor;
		str = str + " to: "+  endFloor;
		str = str + " currently "+ state;
		return str;
	}
	
	/**
	 * This elevator user is a compulsive button presser:
	 * <ul>
	 * <li> Press up and down buttons at the floor </li>
	 * <li> Enter the elevator when it stops at their floor and is travelling in the required direction </li>
	 * <li> Presses all buttons inside the elevator </li>
	 * <li> Leaves the elevator when it arrives at the desired destination floor and presses floor buttons at destination</li>
	 * </ul>
	 */
	public void makeDecision(){
		
		// What direction does the user wish to travel?
		DirectionState user_direction;
		if (startFloor < endFloor) user_direction = DirectionState.UP;
		                      else user_direction = DirectionState.DOWN;
		
		
		// Which floor button(s) does the user press?
       if (state == UserState.ARRIVED_AT_FLOOR){
    	  if (startFloor != 0){
    		  elevator.pressDown(startFloor); 

    	  }
    	  if (startFloor != elevator.topFloor()){
    		  elevator.pressUp(startFloor);

    	  }
    	 pressedButtonAtFloor();
       }
	
		
		// Does the user get into the elevator and what buttons do they press?
       else if (startFloor == elevator.currentFloor() && 
    		 state == UserState.WAITING_AT_FLOOR &&
    		 user_direction == elevator.currentDirection() 
    		 ){
    			 pickedUpAtFloor();
    			 for (int floor = 0; floor<= elevator.topFloor(); floor++) elevator.pressFloorInElevator(floor);
    		 }
		 
		 // Does the user get out of the elevator and what buttons do they press?
       else  if (endFloor== elevator.currentFloor() && state  == UserState.IN_ELEVATOR){ 
        	 deliveredToFloor();
             if (endFloor != 0) elevator.pressDown(endFloor); 
   	         if (endFloor != elevator.topFloor()) elevator.pressUp(endFloor);
             }
    	 }
			

}
	


