package models.elevators;

import interfaces.ElevatorWithDoorsSpecification;
import models.elevatorDoors.DoorsOnFloors;
import tools.HasInvariant;

/**
 * @author jpaulgibson
 */
public class ElevatorWithDoors extends Elevator implements HasInvariant, ElevatorWithDoorsSpecification{
	
	DoorsOnFloors doors;
	
	private void initialise_doors(int numberOfFloors){
		
        doors = new DoorsOnFloors(numberOfFloors);

		for (int floor = 0; floor< numberOfFloors; floor++)
			doors.closeDoor(floor);
		
		doors.openDoor(currentFloor);
	}
	
   public ElevatorWithDoors(int numberOfFloors){
		
		super(numberOfFloors);
		
		initialise_doors(numberOfFloors);
	}
	
	public ElevatorWithDoors(int numberOfFloors,int delay_time){
		
		super(numberOfFloors, delay_time);
		
		initialise_doors(numberOfFloors);
		
	}
	
	public boolean doorOpenAtFloor(int floor){
		
		return doors.isDoorOpen(floor);
	}
	
	@Override
	public boolean updateFloor (){
	
		
		if (doors.isDoorOpen(currentFloor)) doors.closeDoor(currentFloor);
		boolean stopAtNextFloor = super.updateFloor();
		if (stopAtNextFloor) doors.openDoor(currentFloor);
		return stopAtNextFloor;
		
	}
	
	
	public String toString (){
		String str = super.toString();
		str = str+"\n"+ doors.toString();
		
		if (!invariant()) str = str+ "\n **** WARNING - Invariant Broken ***** \n";
		   
		   
		return str;
	}

	/**
	 * Doors can only be open at current floor
	 */
	@Override
	public boolean invariant() {
		
		if (!doors.invariant()) return false; // doors are broken
		if (!super.invariant()) return false; // lift broken
		
		//  Safety invariant doors can only be open at current floor
		
	    for (int fl=0; fl<NUMBER_OF_FLOORS; fl++)
	    	if (doors.isDoorOpen(fl) && currentFloor != fl) return false;
		
		return true;
	}

}
