package models.elevators;

import interfaces.ElevatorWithLockableFloorsSpecification;
import enumerations.SecurityType;
import models.elevatorButtons.LockableFloorButtonsInElevator_Option1;
import models.elevatorButtons.LockableFloorButtonsInElevator_Option2;
import abstractions.LockableFloorButtonsInElevator;

/**
 * This class provides an optional extension with a choice of 2 behaviours:
 * <ul>
 * <li> Option 1 - {@link LockableFloorButtonsInElevator_Option1}
 * <li> Option 2 - {@link LockableFloorButtonsInElevator_Option2}
 * </ul>
 * @author jpaulgibson
 */
public class ElevatorWithLockableFloors  extends Elevator implements ElevatorWithLockableFloorsSpecification{
	
	
	public ElevatorWithLockableFloors(int numberOfFloors, boolean[] lockablefloorButtons, SecurityType option, int delay) 
			throws IllegalArgumentException {
		super(numberOfFloors, delay);
		
		
		if (option == SecurityType.OPTION1)
	 	this.floorButtons = new LockableFloorButtonsInElevator_Option1(numberOfFloors, lockablefloorButtons);
		
		if (option == SecurityType.OPTION2)
        this.floorButtons = new LockableFloorButtonsInElevator_Option2(numberOfFloors, lockablefloorButtons);
	}	
	
	public ElevatorWithLockableFloors(int numberOfFloors, boolean[] lockablefloorButtons, SecurityType option) 
			throws IllegalArgumentException {
		super(numberOfFloors);
		
		
		if (option == SecurityType.OPTION1)
	 	this.floorButtons = new LockableFloorButtonsInElevator_Option1(numberOfFloors, lockablefloorButtons);
		
		if (option == SecurityType.OPTION2)
        this.floorButtons = new LockableFloorButtonsInElevator_Option1(numberOfFloors, lockablefloorButtons);
	}	
	
	
	
	public void lock (int floor){
		((LockableFloorButtonsInElevator) floorButtons).lock(floor);
	}
	
	public void unlock (int floor){
	   ((LockableFloorButtonsInElevator) floorButtons).unlock(floor);
	}
	
	public boolean is_lockable(int floor){
		
		return ((LockableFloorButtonsInElevator) floorButtons).is_lockable(floor);
		
	}
	
	public boolean is_locked(int floor){
		
		return ((LockableFloorButtonsInElevator)floorButtons).is_locked(floor);
	}
}
