package simulations.graphical;


import simulations.Users_Simulation;
import tools.DateHeader;
import tools.SeedRNGCommandLine;
import views.JFrame_ElevatorSpecification;
import enumerations.LogicType;
import enumerations.SecurityType;
import enumerations.UserType;
import factoriesSPL.ElevatorFactory;
import interfaces.ElevatorSpecification;

import java.util.Random;

/**
 * A GUI simulation of an elevator, where a users simulation
 *  - as implemented in {@link simulations.Users_Simulation} - allows simulation of the elevator 
 *  users' button presses. <br><br>
 *  
 *  The simulation is easily re-configured by changing the constant variables
 * @author jpaulgibson
 * @see simulations.graphical.Elevator_GUI_KeyboardController
 */
public class Elevator_GUI_FullSimulation {
	
	/**
	  * Number of floors
	  */
	 final static int NUMBER_OF_FLOORS = 8;
	 
	 /**
	  * Logic controller type in elevator
	  */
	 static LogicType CONTROLLER = LogicType.STUPID;
   
   /**
    * The security type of the floors on the elevator
    */
   final static SecurityType SECURITY = SecurityType.NONE;
   
	 /**
	  * The type of user to use in the simulation
	  */
	 final static UserType USER_TYPE = UserType.PRESS_ALL_BUTTONS;
	 
	 /**
	  * Doors at floors?
	  */
	 final static boolean DOORS = true;
   
	 
	 /**
	  * Frequency at which new users arrive
	  */
	 final static int USER_ARRIVAL_FREQUENCY = 2000; // ms
	 
	 /**
	  * Simulation time 
	  */
	 final static int SIMULATION_TIME = 50000; //ms
	 
     // Configuration Parameters for the animation
		
			final static int TIME_BETWEEN_UPDATES = 1000; // ms
			final static int PERCENTAGE_OF_TIME_BETWEEN_FLOORS = 50; // Should be between 10 and 90
			final static int TIME_BETWEEN_FLOORS =  TIME_BETWEEN_UPDATES * PERCENTAGE_OF_TIME_BETWEEN_FLOORS /100;
			
	 // Configuration Parameters for the frame in which the animation is running
			
			final static int TOP_LEFT_FRAME_HORIZONTAL_COORD = 600; // distance in pixels from left of screen
			final static int TOP_LEFT_FRAME_VERTICAL_COORD = 30; // distance in pixels from top of screen
   

        final static String FRAME_TITLE = "Elevator_GUI_FullSimulation";

	
	public static void main(String[] args){
      
		ElevatorSpecification elevator = 
				 ElevatorFactory.makeElevator(DOORS, NUMBER_OF_FLOORS, CONTROLLER, SECURITY, TIME_BETWEEN_UPDATES);
	   	 
		Random rng = SeedRNGCommandLine.getRandom(args);
		     System.out.println(DateHeader.dateString());
		     
		@SuppressWarnings("unused")
			JFrame_ElevatorSpecification elevator_frame = 
			 new JFrame_ElevatorSpecification(elevator,
					                          FRAME_TITLE, 
								               TOP_LEFT_FRAME_HORIZONTAL_COORD, 
								               TOP_LEFT_FRAME_VERTICAL_COORD,  
								               TIME_BETWEEN_FLOORS);
				
		
		
		Users_Simulation simulationUsers = new Users_Simulation(rng,elevator,USER_ARRIVAL_FREQUENCY,USER_TYPE);

		
		System.out.println("Graphical Simulation Started");
		simulationUsers.animation_on();
	    elevator.elevator_logic_unit_on();
		
		try {
			Thread.sleep(SIMULATION_TIME);
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
		
		simulationUsers.animation_off();
	    elevator.elevator_logic_unit_off();
	    System.out.println("Graphical Simulation Finished");
		
	}

}
