package simulations.graphical;

import views.JFrame_ElevatorSpecification;
import controllers.KeyboardButtonsForElevator;
import enumerations.LogicType;
import enumerations.SecurityType;
import factoriesSPL.ElevatorFactory;
import interfaces.ElevatorSpecification;

/**
 * A GUI simulation of an elevator, where a keyboard controller 
 *  - as implemented in {@link controllers.KeyboardButtonsForElevator} - allows simulation of the elevator 
 *  users' button presses. <br><br>
 *  
 *  The simulation is easily re-configured by changing the constant variables
 * @author jpaulgibson
 * @see simulations.graphical.Elevator_GUI_FullSimulation
 */
public class Elevator_GUI_KeyboardController {
	
	final static String FRAME_TITLE = "KeyboardButtonsForElevator Simulation Frame";
	final static int TOP_LEFT_FRAME_HORIZONTAL_COORD = 100; 
	final static  int TOP_LEFT_FRAME_VERTICAL_COORD = 100;  
	final static  int ANIMATION_TIME_BETWEEN_FLOORS = 1000;
	final static int TIME_RESTING_AT_FLOORS = 500;
	
	final static int TIME_BETWEEN_FLOOR_UPDATES = ANIMATION_TIME_BETWEEN_FLOORS + TIME_RESTING_AT_FLOORS;
	
	final static boolean DOORS = true;
	final static int NUMBER_OF_FLOORS = 6;
	final static LogicType LOGIC_TYPE = LogicType.SMART;
	final static SecurityType SECURITY_TYPE = SecurityType.NONE;
	
	public static void main(String[] args) {
		
		ElevatorSpecification elevator = ElevatorFactory.makeElevator(DOORS, NUMBER_OF_FLOORS, LOGIC_TYPE, SECURITY_TYPE, TIME_BETWEEN_FLOOR_UPDATES);
		
		JFrame_ElevatorSpecification elevator_frame = 
		 new JFrame_ElevatorSpecification(elevator,
				                          FRAME_TITLE, 
							               TOP_LEFT_FRAME_HORIZONTAL_COORD, 
							               TOP_LEFT_FRAME_VERTICAL_COORD,  
							               ANIMATION_TIME_BETWEEN_FLOORS);

	@SuppressWarnings("unused")
	KeyboardButtonsForElevator keyboard = new KeyboardButtonsForElevator(elevator, elevator_frame );
	
	elevator.elevator_logic_unit_on();
		
	}
	

}
