package unit_tests.elevatorUsers;


import org.junit.Assert;
import org.junit.Test;

import unit_tests.elevatorUsers.unitTestAbstractions_elevatorUsers.JUnit_ElevatorUserSpecification;
import models.elevatorUsers.IntelligentElevatorUser;
import enumerations.DirectionState;
import enumerations.UserState;

/**
 * Unit test for {@link models.elevatorUsers.IntelligentElevatorUser}<br>
 * Extends {@link JUnit_ElevatorUserSpecification}
 * 
 * @author jpaulgibson
 * @see validation_tests.Validation_IntelligentElevatorUser
 */
public class JUnit_IntelligentElevatorUser extends JUnit_ElevatorUserSpecification {
	

	
	
	@Override
	public   void setUp() {
		
		elevatorUserUnderTest = new IntelligentElevatorUser(USER_FROM_FLOOR, USER_TO_FLOOR, elevator);
		
		Assert.assertTrue(elevatorUserUnderTest.invariant());
	}


	@Override
	@Test
	public void test_constructionIDCounter(){
		int NUMBER_OF_CONSTRUCTION_TESTS =10;
		
		int count = elevatorUserUnderTest.hashCode();
		
		for (int construction=0; construction < NUMBER_OF_CONSTRUCTION_TESTS; construction++){
	    	elevatorUserUnderTest = new IntelligentElevatorUser(USER_FROM_FLOOR, USER_TO_FLOOR, elevator);
	    	Assert.assertEquals(count+construction+1,elevatorUserUnderTest.hashCode());
			Assert.assertTrue(elevatorUserUnderTest.invariant());
		}
		
	}
	
	@Override
	@Test(expected=IllegalArgumentException.class)
	public void test_constructionKO1(){

	    	elevatorUserUnderTest = new IntelligentElevatorUser(0, 0, elevator);
		}
		
	@Override
	@Test(expected=IllegalArgumentException.class)
	public void test_constructionKO2(){

	    	elevatorUserUnderTest = new IntelligentElevatorUser(-1, 0, elevator);
		}
	
	@Override
	@Test(expected=IllegalArgumentException.class)
	public void test_constructionKO3(){

	    	elevatorUserUnderTest = new IntelligentElevatorUser(0, -1, elevator);
		}


	@Override
	public void test_equalsHash() {
		
		IntelligentElevatorUser user2 = new IntelligentElevatorUser(USER_FROM_FLOOR, USER_TO_FLOOR, elevator);
		
		Assert.assertNotEquals(user2.hashCode(), elevatorUserUnderTest.hashCode());
		
		IntelligentElevatorUser user3 = (IntelligentElevatorUser) elevatorUserUnderTest;

		Assert.assertNotEquals(elevatorUserUnderTest, null);
		Assert.assertNotEquals(elevatorUserUnderTest, "aaaa");
		Assert.assertEquals(elevatorUserUnderTest, user3);
		Assert.assertNotEquals(elevatorUserUnderTest,user2);

	}


	@Override
	@Test
	public  void test_toString(){
		Assert.assertEquals("IntelligentElevatorUser "+elevatorUserUnderTest.hashCode()+" - from: "
				+ USER_FROM_FLOOR +" to: "+ USER_TO_FLOOR+ " currently ARRIVED_AT_FLOOR",
	            elevatorUserUnderTest.toString());
		
		Assert.assertTrue(elevatorUserUnderTest.invariant());
	}
 
	@Override
	public void test_makeDecision() {
		

		// No buttons pressed when user arrives at floor
	
		Assert.assertEquals(UserState.ARRIVED_AT_FLOOR, elevatorUserUnderTest.get_state());
		
		for (int floor = 0; floor < elevator.topFloor(); floor++){
			
	          Assert.assertFalse(elevator.floorPressedInElevator(floor));
	          Assert.assertFalse(elevator.upPressedAtFloor(floor));
	          Assert.assertFalse(elevator.downPressedAtFloor(floor));
			}
		
		// The elevator is not at the floor of the user so the user makes a decision to press the up button
		
		Assert.assertNotEquals(elevator.currentFloor(), USER_FROM_FLOOR);
		elevatorUserUnderTest.makeDecision();
		
		// Check that the user has pressed only 1 button on the floor
		for (int floor = 0; floor < elevator.topFloor(); floor++){
					
			Assert.assertFalse(elevator.floorPressedInElevator(floor));
		   if (floor == USER_FROM_FLOOR) Assert.assertTrue(elevator.upPressedAtFloor(floor));
		                            else Assert.assertFalse(elevator.upPressedAtFloor(floor));
			Assert.assertFalse(elevator.downPressedAtFloor(floor));
		}
		
		// Check that the user is now waiting at the floor
		Assert.assertEquals(UserState.WAITING_AT_FLOOR, elevatorUserUnderTest.get_state());
		
		// The elevator decides to move UP but has not yet moved
		elevator.updateFloor();	

		Assert.assertEquals(DirectionState.UP, elevator.currentDirection());
		Assert.assertNotEquals(elevator.currentFloor(), USER_FROM_FLOOR); 
		
		// Check that the user has not pressed any additional buttons
		for (int floor = 0; floor < elevator.topFloor(); floor++){
					
			Assert.assertFalse(elevator.floorPressedInElevator(floor));
		   if (floor == USER_FROM_FLOOR) Assert.assertTrue(elevator.upPressedAtFloor(floor));
		                            else Assert.assertFalse(elevator.upPressedAtFloor(floor));
			Assert.assertFalse(elevator.downPressedAtFloor(floor));
		}

		// The elevator arrives at the user's floor and the user should make the decision to go in
		elevator.updateFloor(); 
		elevatorUserUnderTest.makeDecision();
		
		Assert.assertEquals(UserState.IN_ELEVATOR, elevatorUserUnderTest.get_state());
		
		// Check that the user has pressed only a single target floor inside the elevator
		for (int floor = 0; floor < elevator.topFloor(); floor++){
			
			if (floor == USER_TO_FLOOR) Assert.assertTrue(elevator.floorPressedInElevator(floor));
            else Assert.assertFalse(elevator.floorPressedInElevator(floor));
					
			Assert.assertFalse(elevator.upPressedAtFloor(floor));
		   
			Assert.assertFalse(elevator.downPressedAtFloor(floor));
		}
		
	    // The elevator moves up and the user should make the decision to get off
		elevator.updateFloor();
	    elevatorUserUnderTest.makeDecision();
	   
	    // There should be no buttons pressed
		for (int floor = 0; floor < elevator.topFloor(); floor++){
			
	          Assert.assertFalse(elevator.floorPressedInElevator(floor));
	          Assert.assertFalse(elevator.upPressedAtFloor(floor));
	          Assert.assertFalse(elevator.downPressedAtFloor(floor));
			}
		Assert.assertEquals(UserState.SERVICED, elevatorUserUnderTest.get_state());
			
	}

}
