package unit_tests.elevatorUsers;

import org.junit.Assert;
import org.junit.Test;

import unit_tests.elevatorUsers.unitTestAbstractions_elevatorUsers.JUnit_ElevatorUserSpecification;
import models.elevatorUsers.PressAllButtonsElevatorUser;
import enumerations.DirectionState;
import enumerations.UserState;

/**
 * Unit test for {@link models.elevatorUsers.PressAllButtonsElevatorUser}<br>
 * Extends {@link JUnit_ElevatorUserSpecification}
 * 
 * @author jpaulgibson
 * @see validation_tests.Validation_PressAllButtonsElevatorUser
 */
public class JUnit_PressAllButtonsElevatorUser extends JUnit_ElevatorUserSpecification {
	
	
	@Override
	public   void setUp() {
		
		elevatorUserUnderTest = new PressAllButtonsElevatorUser(USER_FROM_FLOOR, USER_TO_FLOOR, elevator);
		
		Assert.assertTrue(elevatorUserUnderTest.invariant());
	}


	@Override
	@Test
	public void test_constructionIDCounter(){
		int NUMBER_OF_CONSTRUCTION_TESTS =10;
		
		int count = elevatorUserUnderTest.hashCode();
		
		for (int construction=0; construction < NUMBER_OF_CONSTRUCTION_TESTS; construction++){
	    	elevatorUserUnderTest = new PressAllButtonsElevatorUser(USER_FROM_FLOOR, USER_TO_FLOOR, elevator);
	    	Assert.assertEquals(count+construction+1,elevatorUserUnderTest.hashCode());
			Assert.assertTrue(elevatorUserUnderTest.invariant());
		}
		
	}
	
	@Override
	@Test(expected=IllegalArgumentException.class)
	public void test_constructionKO1(){

	    	elevatorUserUnderTest = new PressAllButtonsElevatorUser(0, 0, elevator);
		}
		
	@Override
	@Test(expected=IllegalArgumentException.class)
	public void test_constructionKO2(){

	    	elevatorUserUnderTest = new PressAllButtonsElevatorUser(-1, 0, elevator);
		}
	
	@Override
	@Test(expected=IllegalArgumentException.class)
	public void test_constructionKO3(){

	    	elevatorUserUnderTest = new PressAllButtonsElevatorUser(0, -1, elevator);
		}


	@Override
	public void test_equalsHash() {
		
		PressAllButtonsElevatorUser user2 = new PressAllButtonsElevatorUser(USER_FROM_FLOOR, USER_TO_FLOOR, elevator);
		
		Assert.assertNotEquals(user2.hashCode(), elevatorUserUnderTest.hashCode());
		
		PressAllButtonsElevatorUser user3 = (PressAllButtonsElevatorUser) elevatorUserUnderTest;

		Assert.assertNotEquals(elevatorUserUnderTest, null);
		Assert.assertNotEquals(elevatorUserUnderTest, "aaaa");
		Assert.assertEquals(elevatorUserUnderTest, user3);
		Assert.assertNotEquals(elevatorUserUnderTest,user2);
		
	}


	@Override
	@Test
	public  void test_toString(){
		Assert.assertEquals("PressAllButtonsElevatorUser "+elevatorUserUnderTest.hashCode()+" - from: "
				+ USER_FROM_FLOOR +" to: "+ USER_TO_FLOOR+ " currently ARRIVED_AT_FLOOR",
				            elevatorUserUnderTest.toString());
		
		Assert.assertTrue(elevatorUserUnderTest.invariant());
	}


	@Override
	public void test_makeDecision() {
		
		Assert.assertFalse(elevator.upPressedAtFloor(USER_FROM_FLOOR ));
		Assert.assertFalse(elevator.downPressedAtFloor(USER_FROM_FLOOR ));
		Assert.assertEquals(0, elevator.currentFloor());
		Assert.assertEquals(DirectionState.STAY, elevator.currentDirection());
		Assert.assertEquals(UserState.ARRIVED_AT_FLOOR, elevatorUserUnderTest.get_state());
		
		elevatorUserUnderTest.makeDecision();
		
		Assert.assertTrue(elevator.upPressedAtFloor(USER_FROM_FLOOR ));
		Assert.assertTrue(elevator.downPressedAtFloor(USER_FROM_FLOOR ));
		Assert.assertEquals(0, elevator.currentFloor());
		Assert.assertEquals(DirectionState.STAY, elevator.currentDirection());
		Assert.assertEquals(UserState.WAITING_AT_FLOOR, elevatorUserUnderTest.get_state());

		elevator.updateFloor();
		
		Assert.assertTrue(elevator.upPressedAtFloor(USER_FROM_FLOOR ));
		Assert.assertTrue(elevator.downPressedAtFloor(USER_FROM_FLOOR ));
		Assert.assertEquals(0, elevator.currentFloor());
		Assert.assertEquals(DirectionState.UP, elevator.currentDirection());
		Assert.assertEquals(UserState.WAITING_AT_FLOOR, elevatorUserUnderTest.get_state());
		
		elevator.updateFloor(); elevatorUserUnderTest.makeDecision();
		
		Assert.assertFalse(elevator.upPressedAtFloor(USER_FROM_FLOOR ));
		Assert.assertTrue(elevator.downPressedAtFloor(USER_FROM_FLOOR ));
		Assert.assertEquals(1, elevator.currentFloor());
		Assert.assertEquals(DirectionState.UP, elevator.currentDirection());
		for (int floor =0; floor< elevator.topFloor(); floor++)
			 Assert.assertTrue(elevator.floorPressedInElevator(floor));
		Assert.assertEquals(UserState.IN_ELEVATOR, elevatorUserUnderTest.get_state());
		
		elevator.updateFloor(); elevatorUserUnderTest.makeDecision();
			
		Assert.assertEquals(elevator.currentFloor(), USER_TO_FLOOR);
		Assert.assertTrue(elevator.upPressedAtFloor(USER_TO_FLOOR));
		Assert.assertTrue(elevator.downPressedAtFloor(USER_TO_FLOOR)); 
		Assert.assertEquals(UserState.SERVICED, elevatorUserUnderTest.get_state());
	
	}

}

