package unit_tests.elevators;

import org.junit.Assert;
import org.junit.Test;

import enumerations.DirectionState;
import models.elevatorLogicUnit.EmergencyLogicUnit;
import models.elevators.Elevator;
import unit_tests.elevators.unitTestAbstractions_elevators.JUnit_ElevatorSpecification;
import validation_tests.use_case10.Validation_Elevator_UseCase10;

public  class JUnit_Elevator_Emergency  extends JUnit_ElevatorSpecification{

	@Override
	public void setUp() throws Exception {
		        
		        // Make an emergency controller, not yet bound to (installed in) a particular elevator
		         EmergencyLogicUnit controller = new EmergencyLogicUnit(null);
				
				//Make an elevator without doors, not yet bound to a particular controller
				elevatorUnderTest = new Elevator(Validation_Elevator_UseCase10.MINIMUM_NUMBER_OF_FLOORS);

				// Install the controller to be tested in the elevator
				elevatorUnderTest.installLogicUnit(controller);
		
	}
	
	
	/**
	 * The use case scenario 1, expected final state :
	 *   Elevator State expected after running scenario:
         DirectionState = STAY
         Floor = 0
         Number of floors = 5
         Requested Floors Inside Elevator: 1 4
         DirectionState Requests For Elevator At Floor: 0- 1- 2-UD 3-D 4-
         Controller type = Emergency Controller

	 */
	@Test
	public void test_useCaseScenario1(){
		
        Validation_Elevator_UseCase10.validate(elevatorUnderTest);
		
		Assert.assertEquals(elevatorUnderTest.currentDirection(), DirectionState.STAY);
		Assert.assertEquals(elevatorUnderTest.currentFloor(), 0);
		

		   Assert.assertFalse(elevatorUnderTest.floorPressedInElevator(0));
		   Assert.assertFalse(elevatorUnderTest.downPressedAtFloor(0));
		   Assert.assertFalse(elevatorUnderTest.upPressedAtFloor(0));
		   
		   Assert.assertTrue(elevatorUnderTest.floorPressedInElevator(1));
		   Assert.assertFalse(elevatorUnderTest.downPressedAtFloor(1));
		   Assert.assertFalse(elevatorUnderTest.upPressedAtFloor(1));
		   
		   Assert.assertFalse(elevatorUnderTest.floorPressedInElevator(2));
		   Assert.assertTrue(elevatorUnderTest.downPressedAtFloor(2));
		   Assert.assertTrue(elevatorUnderTest.upPressedAtFloor(2));
		   
		   Assert.assertFalse(elevatorUnderTest.floorPressedInElevator(3));
		   Assert.assertTrue(elevatorUnderTest.downPressedAtFloor(3));
		   Assert.assertFalse(elevatorUnderTest.upPressedAtFloor(3));
		   
		   Assert.assertTrue(elevatorUnderTest.floorPressedInElevator(4));
		   Assert.assertFalse(elevatorUnderTest.downPressedAtFloor(4));
		   Assert.assertFalse(elevatorUnderTest.upPressedAtFloor(4));

	
	}

}