package unit_tests.factoriesSPL;

import enumerations.LogicType;
import factoriesSPL.LogicUnitFactory;
import interfaces.LogicUnitForElevatorSpecification;

import java.util.Random;
import org.junit.Assert;
import org.junit.Test;


/**
 * Unit tests for constructing a range of different elevator logic units as permitted by the
 * {@link LogicUnitFactory}
 * @author jpaulgibson
 * @see validation_tests.validation_factoriesSPL.Validation_LogicUnitFactory
 */
public class JUnit_LogicUnitFactory {
	
	/**
	 * Unit test for {@link factoriesSPL.LogicUnitFactory#makeLogicUnit(Random ) } exception case
	 * 
	 */
	@Test(expected = IllegalArgumentException.class)
	public void test_makeRandomController(){
		
		Random rng = null;
		
		@SuppressWarnings("unused")
		LogicUnitForElevatorSpecification controller = LogicUnitFactory.makeLogicUnit(rng);
	}
	
	/**
	 * Unit test for {@link factoriesSPL.LogicUnitFactory#makeLogicUnit(LogicType ) }
	 * 
	 */
	@Test
	public void test_makeEmergencyController(){
		
		LogicType  controller_type = LogicType.EMERGENCY;
		
		LogicUnitForElevatorSpecification controller = LogicUnitFactory.makeLogicUnit(controller_type);
		Assert.assertEquals(controller_type.toString(), controller.toString());
		
	}
	
	/**
	 * Unit test for {@link factoriesSPL.LogicUnitFactory#makeLogicUnit(LogicType ) }
	 * 
	 */
	@Test
	public void test_makeStupidController(){
		
		LogicType  controller_type = LogicType.STUPID;
		
		LogicUnitForElevatorSpecification controller = LogicUnitFactory.makeLogicUnit(controller_type);
		Assert.assertEquals(controller_type.toString(), controller.toString());
		
	}
	
	
	/**
	 * Unit test for {@link factoriesSPL.LogicUnitFactory#makeLogicUnit(LogicType ) }
	 * 
	 */
	@Test
	public void test_makeImprovedController(){
		
		LogicType  controller_type = LogicType.IMPROVED;
		
		LogicUnitForElevatorSpecification controller = LogicUnitFactory.makeLogicUnit(controller_type);
		Assert.assertEquals(controller_type.toString(), controller.toString());
		
	}
	
	/**
	 * Unit test for {@link factoriesSPL.LogicUnitFactory#makeLogicUnit(LogicType ) }
	 * 
	 */
	@Test
	public void test_makeSmartController(){
		
		LogicType  controller_type = LogicType.SMART;
		
		LogicUnitForElevatorSpecification controller = LogicUnitFactory.makeLogicUnit(controller_type);
		Assert.assertEquals(controller_type.toString(), controller.toString());
		
	}

}
