package validation_tests.use_case10;

import enumerations.SecurityType;
import models.elevatorLogicUnit.ImprovedLogicUnit;
import models.elevators.ElevatorWithLockableFloors;
import tools.DateHeader;

/**
 * Validation test for an {@link ElevatorWithLockableFloors} controlled by {@link ImprovedLogicUnit}<br>
 * The use case is defined in the class {@link validation_tests.use_case10.Validation_Elevator_UseCase10}<br><br>
 * For testing the additional secure behaviour:
 * <ul>
 * <li>We make floors 1,2 and 4 lockable
 * <li>We lock floors 1 and 2 and unlock floor 4
 * <li> When we execute the scenario, we should get a warning message about floor 1 being locked and so the
 * floor request not being authorised
 * </ul>
 * 
 * We use the {@link DateHeader} class to document the date/time of the test execution<br><br>
 * Expected output:
<pre>

********************************************************************
Execution Date/Time 2015/07/28 10:41:35
********************************************************************
Floor 1 is locked and your request cannot be authorized.

Elevator State:
DirectionState = UP
Stopping at floor = false
Floor = 0
Number of floors = 5
Requested Floors Inside Elevator: 4
Lockable Floors In Elevator:  1:L 2:L  4:U
DirectionState Requests For Elevator At Floor: 0- 1- 2-UD 3-D 4-
Controller type = Improved Stupid Controller

Elevator State:
DirectionState = UP
Stopping at floor = false
Floor = 1
Number of floors = 5
Requested Floors Inside Elevator: 4
Lockable Floors In Elevator:  1:L 2:L  4:U
DirectionState Requests For Elevator At Floor: 0- 1- 2-UD 3-D 4-
Controller type = Improved Stupid Controller

Elevator State:
DirectionState = UP
Stopping at floor = true
Floor = 2
Number of floors = 5
Requested Floors Inside Elevator: 4
Lockable Floors In Elevator:  1:L 2:L  4:U
DirectionState Requests For Elevator At Floor: 0- 1- 2-D 3-D 4-
Controller type = Improved Stupid Controller

Elevator State:
DirectionState = UP
Stopping at floor = false
Floor = 3
Number of floors = 5
Requested Floors Inside Elevator: 4
Lockable Floors In Elevator:  1:L 2:L  4:U
DirectionState Requests For Elevator At Floor: 0- 1- 2-D 3-D 4-
Controller type = Improved Stupid Controller

Elevator State:
DirectionState = DOWN
Stopping at floor = true
Floor = 4
Number of floors = 5
Requested Floors Inside Elevator:
Lockable Floors In Elevator:  1:L 2:L  4:U
DirectionState Requests For Elevator At Floor: 0- 1- 2-D 3-D 4-
Controller type = Improved Stupid Controller

Elevator State:
DirectionState = DOWN
Stopping at floor = true
Floor = 3
Number of floors = 5
Requested Floors Inside Elevator:
Lockable Floors In Elevator:  1:L 2:L  4:U
DirectionState Requests For Elevator At Floor: 0- 1- 2-D 3- 4-
Controller type = Improved Stupid Controller

Elevator State:
DirectionState = DOWN
Stopping at floor = true
Floor = 2
Number of floors = 5
Requested Floors Inside Elevator:
Lockable Floors In Elevator:  1:L 2:L  4:U
DirectionState Requests For Elevator At Floor: 0- 1- 2- 3- 4-
Controller type = Improved Stupid Controller

Elevator State:
DirectionState = DOWN
Stopping at floor = true
Floor = 2
Number of floors = 5
Requested Floors Inside Elevator:
Lockable Floors In Elevator:  1:L 2:L  4:U
DirectionState Requests For Elevator At Floor: 0- 1- 2- 3- 4-
Controller type = Improved Stupid Controller

The elevator made 4 floor stops until execution stopped
</pre>
 * 
 * @author J Paul Gibson
 */
public class Validation_ElevatorLockableFloorsImproved {
	
	public static void main(String[] args) {
		
		 System.out.println(DateHeader.dateString());
			
			// Make an improved stupid controller, not yet bound to (installed in) a particular elevator
			ImprovedLogicUnit controller = new ImprovedLogicUnit(null);
			
			//Make an elevator with lockable floors, not yet bound to a particular controller
			
			boolean [] lockedFloors = {false, true, true, false, true};
			
			SecurityType security =  SecurityType.OPTION1; // We pass the option parameter to specify which optional behaviour is required
			ElevatorWithLockableFloors elevator = 
					new ElevatorWithLockableFloors(Validation_Elevator_UseCase10.MINIMUM_NUMBER_OF_FLOORS, 
					                               lockedFloors, security);
			
			elevator.unlock(4);

			// Install the controller to be tested in the elevator
			elevator.installLogicUnit(controller);

			// Run the validation validation_tests on the controller by binding it to an elevator
			Validation_Elevator_UseCase10.validate(elevator);
		
	
		
	}


}
