package validation_tests.use_case10;

import interfaces.ElevatorSpecification;
import models.elevatorLogicUnit.SmartLogicUnit;
import models.elevators.ElevatorWithDoors;
import tools.DateHeader;

/**
 * Validation test for an {@link ElevatorWithDoors} controlled by {@link SmartLogicUnit}<br>
 * The use case is defined in the class {@link validation_tests.use_case10.Validation_Elevator_UseCase10}<br><br>
 * We use the {@link DateHeader} class to document the date/time of the test execution<br><br>
 * Expected output:
<pre>
********************************************************************
Execution Date/Time 2015/07/21 15:23:06
********************************************************************

Elevator State:
DirectionState = UP
Stopping at floor = false
Floor = 0
Number of floors = 5
Requested Floors Inside Elevator: 1 4
DirectionState Requests For Elevator At Floor: 0- 1- 2-UD 3-D 4-
Controller type = Smart Controller
DoorState States For Elevator At Floor: 0- 1- 2- 3- 4-

Elevator State:
DirectionState = UP
Stopping at floor = true
Floor = 1
Number of floors = 5
Requested Floors Inside Elevator: 4
DirectionState Requests For Elevator At Floor: 0- 1- 2-UD 3-D 4-
Controller type = Smart Controller
DoorState States For Elevator At Floor: 0- 1-OPEN 2- 3- 4-

Elevator State:
DirectionState = UP
Stopping at floor = true
Floor = 2
Number of floors = 5
Requested Floors Inside Elevator: 4
DirectionState Requests For Elevator At Floor: 0- 1- 2-D 3-D 4-
Controller type = Smart Controller
DoorState States For Elevator At Floor: 0- 1- 2-OPEN 3- 4-

Elevator State:
DirectionState = UP
Stopping at floor = false
Floor = 3
Number of floors = 5
Requested Floors Inside Elevator: 4
DirectionState Requests For Elevator At Floor: 0- 1- 2-D 3-D 4-
Controller type = Smart Controller
DoorState States For Elevator At Floor: 0- 1- 2- 3- 4-

Elevator State:
DirectionState = DOWN
Stopping at floor = true
Floor = 4
Number of floors = 5
Requested Floors Inside Elevator:
DirectionState Requests For Elevator At Floor: 0- 1- 2-D 3-D 4-
Controller type = Smart Controller
DoorState States For Elevator At Floor: 0- 1- 2- 3- 4-OPEN

Elevator State:
DirectionState = DOWN
Stopping at floor = true
Floor = 3
Number of floors = 5
Requested Floors Inside Elevator:
DirectionState Requests For Elevator At Floor: 0- 1- 2-D 3- 4-
Controller type = Smart Controller
DoorState States For Elevator At Floor: 0- 1- 2- 3-OPEN 4-

Elevator State:
DirectionState = DOWN
Stopping at floor = true
Floor = 2
Number of floors = 5
Requested Floors Inside Elevator:
DirectionState Requests For Elevator At Floor: 0- 1- 2- 3- 4-
Controller type = Smart Controller
DoorState States For Elevator At Floor: 0- 1- 2-OPEN 3- 4-

Elevator State:
DirectionState = DOWN
Stopping at floor = true
Floor = 2
Number of floors = 5
Requested Floors Inside Elevator:
DirectionState Requests For Elevator At Floor: 0- 1- 2- 3- 4-
Controller type = Smart Controller
DoorState States For Elevator At Floor: 0- 1- 2-OPEN 3- 4-

The elevator made 5 floor stops until execution stopped

</pre>
**/

public class Validation_ElevatorWithDoorsSmart {
	
	public static void main(String[] args) {
		
		 System.out.println(DateHeader.dateString());
			
			// Make an improved stupid controller, not yet bound to (installed in) a particular elevator
			SmartLogicUnit controller = new SmartLogicUnit(null);
			
			//Make an elevator without doors, not yet bound to a particular controller
			ElevatorSpecification elevator = new ElevatorWithDoors(Validation_Elevator_UseCase10.MINIMUM_NUMBER_OF_FLOORS);

			// Install the controller to be tested in the elevator
			elevator.installLogicUnit(controller);

			// Run the validation validation_tests on the controller by binding it to an elevator
			Validation_Elevator_UseCase10.validate(elevator);
		
	
		
	}

}
