package validation_tests.use_case2;

import models.elevatorLogicUnit.EmergencyLogicUnit;
import models.elevatorLogicUnit.StupidLogicUnit;
import models.elevators.Elevator;


/**
 * Validation test for an {@link Elevator} controlled by {@link EmergencyLogicUnit}<br>
 * Such an elevator should STAY at the current floor during the emergency.<br>
 * In our test, we replace a stupid controller ({@link StupidLogicUnit}) by an emergency controller whilst the stupid
 * controller is in the middle of servicing the elevator requests.<br>
 * Expected output:
 * <pre>

Elevator State:
DirectionState = STAY
Floor = 0
Number of floors = 5
Requested Floors Inside Elevator: 0 4
DirectionState Requests For Elevator At Floor: 0- 1- 2-D 3-U 4-
Controller type = Stupid Controller
Stopping at next floor

Elevator State:
DirectionState = UP
Floor = 1
Number of floors = 5
Requested Floors Inside Elevator: 0 4
DirectionState Requests For Elevator At Floor: 0- 1- 2-D 3-U 4-
Controller type = Stupid Controller
Stopping at next floor

Elevator State:
DirectionState = STAY
Floor = 1
Number of floors = 5
Requested Floors Inside Elevator: 0 4
DirectionState Requests For Elevator At Floor: 0- 1- 2-D 3-U 4-
Controller type = Emergency Controller
Stopping at next floor

Elevator State:
DirectionState = STAY
Floor = 1
Number of floors = 5
Requested Floors Inside Elevator: 0 4
DirectionState Requests For Elevator At Floor: 0- 1- 2-D 3-U 4-
Controller type = Emergency Controller
Stopping at next floor

Elevator State:
DirectionState = DOWN
Floor = 0
Number of floors = 5
Requested Floors Inside Elevator: 4
DirectionState Requests For Elevator At Floor: 0- 1- 2-D 3-U 4-
Controller type = Stupid Controller
Stopping at next floor

Elevator State:
DirectionState = UP
Floor = 1
Number of floors = 5
Requested Floors Inside Elevator: 4
DirectionState Requests For Elevator At Floor: 0- 1- 2-D 3-U 4-
Controller type = Stupid Controller
</pre>
 * @author J Paul Gibson
 */
public class Validation_ElevatorTemporaryEmergency {

	
	public static void main(String[] args) {
		
		Elevator elevator = new Elevator(5);
		EmergencyLogicUnit emergencyController = new EmergencyLogicUnit(null);
		StupidLogicUnit stupidController = new StupidLogicUnit(null);
		elevator.installLogicUnit(stupidController);
		
		elevator.pressUp(3);
		elevator.pressDown(2);
		elevator.pressFloorInElevator(0);
		elevator.pressFloorInElevator(4);
		System.out.println(elevator);
		
		   if (elevator.updateFloor()) System.out.println("Stopping at next floor");
		                          else System.out.println("Passing  next floor");
		   System.out.println(elevator);
		   
		   elevator.installLogicUnit(emergencyController);
		   
		   if (elevator.updateFloor()) System.out.println("Stopping at next floor");
           else System.out.println("Passing  next floor");
           System.out.println(elevator);
           
		   if (elevator.updateFloor()) System.out.println("Stopping at next floor");
           else System.out.println("Passing  next floor");
           System.out.println(elevator);
           
           elevator.installLogicUnit(stupidController);
		   
		   if (elevator.updateFloor()) System.out.println("Stopping at next floor");
           else System.out.println("Passing  next floor");
           System.out.println(elevator);
		   
		   if (elevator.updateFloor()) System.out.println("Stopping at next floor");
           else System.out.println("Passing  next floor");
           System.out.println(elevator);
		}
		

}
