package views;

import interfaces.ElevatorObserverSpecification;
import interfaces.ElevatorSpecification;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;

import javax.swing.JPanel;

import tools.Ticker;
import tools.TickerObserverSpecification;
import enumerations.DirectionState;

@SuppressWarnings("serial")
public class JPanel_ElevatorSpecification extends JPanel
                                          implements TickerObserverSpecification, ElevatorObserverSpecification{
	
	
	/**
	 * Time (in milliseconds) to move elevator between floors:
	 * should be a postive integer - recommended to be &gt; 1000
	 */
	 final int ANIMATION_TIME; 
	
	 /**
	  * number of animation steps:
	  * should be a positive integer - recommended to be between 5 and 25
	  */
	 final int NUMBER_OF_STEPS = 10;
	 
	 
	 /**
	  * The delay time  between animation steps: 
	  * it should be set to {@link #ANIMATION_TIME} / {@link #NUMBER_OF_STEPS} during construction;
	  */
	 final int DELAY_TIME; 
	 
	 int step = 0; // 0: static view of elevator at floor;  1 - NUMBER_OF_STEPS: dynamic animation step of elevator between floors
	 
	 int previous_floor; // used to decide if the elevator has moved between updates and thus if we need to animate the movement

	 int current_floor;
	 
	 boolean stopAtNextFloor;
	 
	 DirectionState current_direction;
	 
	 
	 
		final static Color BACKGROUND_COLOUR = JPanel_ColourConfiguration.BACKGROUND_COLOUR;
		
		
		final static Color BUTTON_BACKGROUND_CHANGING_COLOUR = JPanel_ColourConfiguration.BUTTON_BACKGROUND_CHANGING_COLOUR;
		final static Color BUTTON_BACKGROUND_OFF_COLOUR = JPanel_ColourConfiguration.BUTTON_BACKGROUND_OFF_COLOUR;
		final static Color BUTTON_BACKGROUND_ON_COLOUR = JPanel_ColourConfiguration.BUTTON_BACKGROUND_ON_COLOUR;
		final static Color BUTTON_TEXT_ON_COLOUR = JPanel_ColourConfiguration.BUTTON_TEXT_ON_COLOUR;
		final static Color BUTTON_TEXT_OFF_COLOUR = JPanel_ColourConfiguration.BUTTON_TEXT_OFF_COLOUR;
		
		final static Color ELEVATOR_COLOUR = JPanel_ColourConfiguration.ELEVATOR_COLOUR;
		final static Color ELEVATOR_TEXT_COLOUR = JPanel_ColourConfiguration.ELEVATOR_TEXT_COLOUR;
		
		final static Color FLOOR_COLOUR = JPanel_ColourConfiguration.FLOOR_COLOUR;
	 
	 
	 
	Ticker ticker_for_animation;

	
	ElevatorSpecification elevator;
	
	
	final int WIDTH = 360;
	final int HEIGHT = 640;
	
	final int FLOOR_WIDTH = 120;
	final int ELEVATOR_WIDTH = WIDTH - FLOOR_WIDTH;
	
	final int NUMBER_OF_FLOORS;
	final int FLOOR_HEIGHT;
	final int ELEVATOR_HEIGHT;
	final int BUTTON_TEXT_SIZE =10;
	final int BUTTON_HEIGHT_IN_LIFT=BUTTON_TEXT_SIZE*2;
	final int BUTTON_HEIGHT_ON_FLOORS =BUTTON_TEXT_SIZE*2;
	
	final Font TEXT_FONT = new Font("Monospaced", Font.BOLD, BUTTON_TEXT_SIZE);
	
	final Dimension size = new Dimension(WIDTH,HEIGHT);
	

	
	public JPanel_ElevatorSpecification(ElevatorSpecification elevator, int animationTime){
		
		super();
		this.elevator = elevator;
		this.ANIMATION_TIME = animationTime;
		DELAY_TIME = ANIMATION_TIME/ NUMBER_OF_STEPS;
		setPreferredSize(size);
		
		NUMBER_OF_FLOORS = elevator.topFloor()+1;
		FLOOR_HEIGHT = HEIGHT/NUMBER_OF_FLOORS;
		ELEVATOR_HEIGHT = FLOOR_HEIGHT/2;
		
		previous_floor = elevator.currentFloor();
		
		ticker_for_animation = new Ticker (DELAY_TIME);
		ticker_for_animation.connect_Observer(this);
		ticker_for_animation.animation_on();
		
		elevator.connect_Observer(this);

	}
		

	
	
	protected void paintFloors(Graphics g){
		// Floors
		for (int floor = 0; floor< NUMBER_OF_FLOORS; floor++){
			g.setColor(FLOOR_COLOUR);
			g.fillRect(	ELEVATOR_WIDTH, HEIGHT - (FLOOR_HEIGHT*floor)-2, FLOOR_WIDTH, 2);
			g.drawString("Floor: "+floor, ELEVATOR_WIDTH, HEIGHT - (FLOOR_HEIGHT*floor)-5);
		}
					
		// UP Buttons
		for (int floor = 0; floor< NUMBER_OF_FLOORS-1; floor++){
			if (elevator.upPressedAtFloor(floor)) g.setColor(BUTTON_BACKGROUND_ON_COLOUR);
			 else {
					 if (current_floor == floor && 
							 stopAtNextFloor   &&
						 current_direction == DirectionState.UP) g.setColor(BUTTON_BACKGROUND_CHANGING_COLOUR); 
					 else g.setColor(BUTTON_BACKGROUND_OFF_COLOUR);
				   }
			g.fillOval(ELEVATOR_WIDTH+60, HEIGHT - (FLOOR_HEIGHT*floor) - BUTTON_HEIGHT_ON_FLOORS , BUTTON_HEIGHT_ON_FLOORS, BUTTON_HEIGHT_ON_FLOORS);
						
			if (elevator.upPressedAtFloor(floor)) g.setColor(BUTTON_TEXT_ON_COLOUR);
				else g.setColor(BUTTON_TEXT_OFF_COLOUR);
			g.drawString("^", ELEVATOR_WIDTH+68, HEIGHT - (FLOOR_HEIGHT*floor)-4);
			}
					
		for (int floor = 1; floor< NUMBER_OF_FLOORS; floor++){
			if (elevator.downPressedAtFloor(floor)) g.setColor(BUTTON_BACKGROUND_ON_COLOUR);
				 else   { if (current_floor == floor && 
							  stopAtNextFloor &&
							  current_direction == DirectionState.DOWN) g.setColor(BUTTON_BACKGROUND_CHANGING_COLOUR); 
						   else
							   g.setColor(BUTTON_BACKGROUND_OFF_COLOUR);
						   }
				g.fillOval(ELEVATOR_WIDTH+70+BUTTON_HEIGHT_ON_FLOORS, HEIGHT - (FLOOR_HEIGHT*floor) - BUTTON_HEIGHT_ON_FLOORS , BUTTON_HEIGHT_ON_FLOORS, BUTTON_HEIGHT_ON_FLOORS);
						
				if (elevator.downPressedAtFloor(floor)) g.setColor(BUTTON_TEXT_ON_COLOUR);
				 else g.setColor(BUTTON_TEXT_OFF_COLOUR);

				g.drawString("v", ELEVATOR_WIDTH+98, HEIGHT - (FLOOR_HEIGHT*floor)-4);
		}	
	
	}
	
	protected void paintElevatorAtFloor(Graphics g){
			
			g.setColor(ELEVATOR_COLOUR);
			g.fillRect(0, HEIGHT - (current_floor*FLOOR_HEIGHT)- ELEVATOR_HEIGHT, ELEVATOR_WIDTH, ELEVATOR_HEIGHT);
			
			g.setColor(ELEVATOR_TEXT_COLOUR);
	        
			g.drawString("["+ elevator.get_logic_unit_Type()+"] On",
					     10,
					     HEIGHT  + ((int)( BUTTON_HEIGHT_IN_LIFT * 1.5)) - (current_floor*FLOOR_HEIGHT)- ELEVATOR_HEIGHT);
		   
			//Elevator Buttons
				
				for (int floor = 0; floor< NUMBER_OF_FLOORS; floor++){
					if (elevator.floorPressedInElevator(floor)) g.setColor(BUTTON_BACKGROUND_ON_COLOUR);
					   else g.setColor(BUTTON_BACKGROUND_OFF_COLOUR);
					g.fillOval(20+ (BUTTON_HEIGHT_ON_FLOORS*floor),
							   HEIGHT - (current_floor*FLOOR_HEIGHT)- ELEVATOR_HEIGHT, 
							   BUTTON_HEIGHT_ON_FLOORS, BUTTON_HEIGHT_ON_FLOORS);
		
					if (elevator.floorPressedInElevator(floor)) g.setColor(BUTTON_TEXT_ON_COLOUR);
					   else g.setColor(BUTTON_TEXT_OFF_COLOUR);
					g.drawString(""+floor, 
							     26+ (BUTTON_HEIGHT_ON_FLOORS*floor),
							     HEIGHT - (current_floor*FLOOR_HEIGHT)- ELEVATOR_HEIGHT +BUTTON_HEIGHT_ON_FLOORS-4);

				}
	}
	
	protected void paintElevatorBetweenFloors(Graphics g){
		
		step++;
		
		 int move_increment = FLOOR_HEIGHT/NUMBER_OF_STEPS;
		 int floor_increment =0;
		 
		 if (previous_floor == current_floor)
			 move_increment = 0;
		 
		 else if (previous_floor > current_floor)
			 floor_increment =1;
		 
		 else { floor_increment =-1;
		     	move_increment =0-move_increment;
		 }
	
		g.setColor(ELEVATOR_COLOUR);
	    g.fillRect(0, 
			   HEIGHT - ((current_floor+floor_increment)*FLOOR_HEIGHT)- ELEVATOR_HEIGHT + move_increment *step, 
				   ELEVATOR_WIDTH, 
				   ELEVATOR_HEIGHT);
	
	for (int floor = 0; floor< NUMBER_OF_FLOORS; floor++){
		if (elevator.floorPressedInElevator(floor)) g.setColor(BUTTON_BACKGROUND_ON_COLOUR);
		   else {
			   if (current_floor == floor && stopAtNextFloor) g.setColor(BUTTON_BACKGROUND_CHANGING_COLOUR); 
			   else
			   g.setColor(BUTTON_BACKGROUND_OFF_COLOUR);
		   }
		g.fillOval(20+ (BUTTON_HEIGHT_ON_FLOORS*floor),
				   HEIGHT - ((current_floor+floor_increment)*FLOOR_HEIGHT)- ELEVATOR_HEIGHT + move_increment *step, 
				   BUTTON_HEIGHT_ON_FLOORS, 
				   BUTTON_HEIGHT_ON_FLOORS);
		
		if (elevator.floorPressedInElevator(floor)) g.setColor(BUTTON_TEXT_ON_COLOUR);
		   else g.setColor(BUTTON_TEXT_OFF_COLOUR);
		g.drawString(""+floor, 
				     26+ (BUTTON_HEIGHT_ON_FLOORS*floor),
				     HEIGHT - ((current_floor+floor_increment)*FLOOR_HEIGHT)- ELEVATOR_HEIGHT + move_increment *step+BUTTON_HEIGHT_ON_FLOORS-4);

	
     }
	}		
	
	@Override
		public void paintComponent(Graphics g){

            g.setColor(getBackground());
       		g.fillRect(0, 0, WIDTH, HEIGHT);
            g.setFont(TEXT_FONT);
			
			paintFloors( g);
			
           if (previous_floor == current_floor){
        	   paintElevatorAtFloor(g);
           }
             else {
            	 paintElevatorBetweenFloors(g);
             }
	}

	
	
	public void start_animation(){	
	
		step =0;
		current_floor  = elevator.currentFloor();
		current_direction = elevator.currentDirection();
		stopAtNextFloor = elevator.stoppingAtNextFloor();
		ticker_for_animation.animation_on();
	}
	

	public void stop_animation(){
	    ticker_for_animation.animation_off();
		step =0;
		previous_floor = current_floor;
		elevator.elevator_logic_unit_on();
		start_animation();
	}

		@Override
		public void observe_ticker() {		
	        repaint();
		    if (step>=NUMBER_OF_STEPS)	stop_animation();
		}
		
		@Override
		public void elevatorUpdated() {
			
			if (step == 0) {
				start_animation();
			}
			
			else elevator.elevator_logic_unit_off();

		}
		
}

